# -*- coding: ISO-8859-1 -*-

# Copyright (C) 2002 Jrg Lehmann <joerg@luga.de>
#
# This file is part of PyTone (http://www.luga.de/pytone/)
#
# PyTone is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2
# as published by the Free Software Foundation.
#
# PyTone is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import copy
import time
import threading

import events

# player states

STOP     = 0
PAUSE    = 1
PLAY     = 2

class playbackinfo:
    
    """ class for storage of playback information

    This class serves as a means of communication between the
    actual players and the player control logic 

    """
    
    def __init__(self, player, state=STOP, song=None, time=0, crossfade=0):
        """ 

        player:    player which this playbackinfo instance refers to
        state:     player state (STOP, PAUSE, PLAY)
        song:      song currently played (or None, if player is not playing)
        time:      position in seconds in the song
        crossfade: crossfade in progress
        """
        self.player = player
        self.state = state
        self.song = song
        self.time = time
        self.crossfade = 0

    def __cmp__(self, other):
        return (cmp(self.player, other.player) or
                cmp(self.state, other.state) or
                cmp(self.song, other.song) or
                cmp(self.time, other.time) or
                cmp(self.crossfade, other.crossfade))

    def __str__(self):
        s = "player %s "
        if self.state==STOP:
            s = s + "stopped"
        elif self.state==PAUSE:
            s = s + "paused"
        elif self.state==PLAY:
            s = s + "playing"
        s = s + " song: "
        if self.song:
            s = s + "%s at time %f" % ( `self.song`, self.time)
        else:
            s = s + "None"
        return s

    __repr__ = __str__

    def updatesong(self, song):
        """ update song and reset time """
        self.song = song
        self.time = 0

    def stopped(self):
        self.state = STOP
        self.song = None
        self.time = 0
        self.crossfade = 0

    def paused(self):
        self.state = PAUSE

    def playing(self):
        self.state = PLAY

    def updatetime(self, time):
        self.time = time

    def updatecrossfade(self, crossfade):
        self.crossfade = crossfade

    def isplaying(self):
        return self.state == PLAY

    def ispaused(self):
        return self.state == PAUSE

    def iscrossfading(self):
        return self.crossfade and not self.state == STOP
        

class genericplayer(threading.Thread):
    def __init__(self, autoplay):
        
        self.autoplay = autoplay

        # the playbackinfo structure describes the current player state
        self.playbackinfo = playbackinfo(self)

        # old playbackinfo, used to detect changes of the player state
        self.oplaybackinfo = copy.copy(self.playbackinfo)
        
        # have we already requested a new song for crossfading?
        self.autorequestednextsong = 0

        # as independent thread, we want our own event channel
        self.channel = events.hub.newchannel()

        self.done = 0

        threading.Thread.__init__(self)

        # we are only servants of higher powers
        self.setDaemon(1)

        self.channel.subscribe(events.playerstart, self.playerstart)
        self.channel.subscribe(events.playerpause, self.playerpause)
        self.channel.subscribe(events.playerstop, self.playerstop)
        self.channel.subscribe(events.playerforward, self.playerforward)
        self.channel.subscribe(events.nextsong, self.nextsong)

    def run(self):
        """main loop"""
        while not self.done:
            self.processevents()
            if self.isplaying():
                self.play()
            self.updatestatus()

    def processevents(self):
        """check for incoming events and process them"""

        if self.isstopped() and self.autoplay:
            self.requestnextsong()
            
        if not self.isplaying():
            time.sleep(0.5)
                
        # process incoming events
        self.channel.process()

    def play(self):
        """play songs

        has to be provided by specialized classes"""
        pass

    def updatestatus(self):
        """notify interested parties of changes in player status"""
        if self.oplaybackinfo!=self.playbackinfo:
            self.oplaybackinfo = copy.copy(self.playbackinfo)
            events.hub.notify(events.playbackinfochanged(self.playbackinfo))

    def playerquit(self):
        """quit player"""
        self.done = 1
        self._playerquit()

    def requestnextsong(self):
        """request next song from playlist, if not already one was requested"""
        if not self.autorequestednextsong:
            self.autorequestednextsong = 1
            events.hub.notify(events.requestnextsong(self))

    def isstopped(self):
        return self.playbackinfo.state==STOP
    
    def ispaused(self):
        return self.playbackinfo.state==PAUSE

    def isplaying(self):
        return self.playbackinfo.state==PLAY

    def _nextsong(self, song):
        """add song to playlist

        has to be provided by specialized classes"""
        pass

    def _playerstart(self):
        """prepare player for playing

        has to be provided by specialized classes"""
        pass

    def _playerpause(self):
        """pause player 

        has to be provided by specialized classes"""
        pass

    def _playerunpause(self):
        """restart player after pause

        has to be provided by specialized classes"""
        pass

    def _playerstop(self):
        """stop playing

        has to be provided by specialized classes"""
        pass
    
    def _playerquit(self):
        """quit player

        has to be provided by specialized classes"""
        pass

    # event handlers

    def playerstart(self, event):
        """start playing"""
        if event.player:
            if self.ispaused():
                self._playerunpause()
            elif self.isstopped():
                self._playerstart()
                self.requestnextsong()
            self.playbackinfo.playing()

    def playerpause(self, event):
        """start/pause player"""
        if event.player is self:
            self.playbackinfo.paused()
            self._playerpause()

    def playerstop(self, event):
        """stop playing"""
        if event.player is self:
            self.autoplay = 0
            self.playbackinfo.stopped()
            self._playerstop()

    def playerforward(self, event):
        """immediately play next song"""
        if event.player is self:
            events.hub.notify(events.requestnextsong(self))

    def nextsong(self, event):
        """play event.song next"""
        if event.player is self:
            if event.song:
                self.playbackinfo.playing()
                self._nextsong(event.song)
                
            # reset autorequestednextsong, but do this after the
            # self._nextsong call, since this method may infer from
            # self.autorequestednextsong whether a manual (via
            # playerforward) or automatic song change has to be
            # performed
            self.autorequestednextsong = 0
